@@ -10,6 +10,7 @@ from typing import Optional
 from typing import Tuple
 
 import requests
+import yaml
 from pystac import STACValidationError
 from pystac_client import Client
 
@@ -126,7 +127,7 @@ invalid_datetimes = [
 
 
 def validate_api(
-    root_url: str, post: bool, conformance_classes: List[str]
+    root_url: str, post: bool, conformance_classes: List[str], collection: str
 ) -> Tuple[List[str], List[str]]:
     warnings: List[str] = []
     errors: List[str] = []
@@ -208,22 +209,23 @@ def validate_api(
 
     if "collections" in conformance_classes:
         print("Validating STAC API - Collections conformance class.")
-        validate_collections(root_body, warnings, errors)
+        validate_collections(root_body, collection, warnings, errors)
 
     if "features" in conformance_classes:
         print("Validating STAC API - Features conformance class.")
+        validate_collections(root_body, collection, warnings, errors)
         validate_features(root_body, conforms_to, warnings, errors)
 
     if "item-search" in conformance_classes:
         print("Validating STAC API - Item Search conformance class.")
-        validate_item_search(root_body, post, conforms_to, warnings, errors)
+        validate_item_search(root_body, post, collection, conforms_to, warnings, errors)
 
     if not errors:
         try:
             catalog = Client.open(root_url)
             catalog.validate()
-            for collection in catalog.get_children():
-                collection.validate()
+            for child in catalog.get_children():
+                child.validate()
         except STACValidationError as e:
             errors.append(f"pystac error: {str(e)}")
 
@@ -259,29 +261,45 @@ def validate_core(
     if not (service_desc := link_by_rel(links, "service-desc")):
         errors.append("/ : Link[rel=service-desc] must exist")
     else:
-        r_service_desc = requests.get(service_desc["href"])
-
-        errors += validate(
-            "/ : Link[service-desc] must return 200",
-            lambda: r_service_desc.status_code == 200,
-        )
-
-        service_desc_type = service_desc.get("type", "")
-        if (
-            (ct := r_service_desc.headers.get("content-type", "")) == service_desc_type
-        ) or (
-            (";" in ct or ";" in service_desc_type)
-            and (ct.split(";", 1)[0] == service_desc_type.split(";", 1)[0])
-        ):
-            pass
+        if not (service_desc_type := service_desc.get("type")):
+            errors.append("/ : Link[rel=service-desc] must have a type defined")
         else:
-            errors.append(
-                f"service-desc ({service_desc}): link must advertise same type as endpoint content-type header, "
-                f"advertised '{service_desc_type}', actually '{ct}'"
+            r_service_desc = requests.get(
+                service_desc["href"], headers={"Accept": service_desc_type}
             )
 
+            if not r_service_desc.status_code == 200:
+                errors.append("/ : Link[service-desc] must return 200")
+            else:
+                content_type = r_service_desc.headers.get("content-type", "")
+                if content_type in ["application/yaml", "application/vnd.oai.openapi"]:
+                    pass
+                    # openapi_spec = r_service_desc.json()
+                    # todo: verify limits exist and test them
+                elif content_type in [
+                    "application/json",
+                    "application/vnd.oai.openapi+json",
+                    "application/vnd.oai.openapi+json;version=3.0",
+                    "application/vnd.oai.openapi+json;version=3.1",
+                ]:
+                    yaml.safe_load(r_service_desc.text)
+
+            if (
+                (ct := r_service_desc.headers.get("content-type", ""))
+                == service_desc_type
+            ) or (
+                (";" in ct or ";" in service_desc_type)
+                and (ct.split(";", 1)[0] == service_desc_type.split(";", 1)[0])
+            ):
+                pass
+            else:
+                errors.append(
+                    f"service-desc ({service_desc}): media type used in Accept header must get response with same "
+                    f"Content-Type header: used '{service_desc_type}', got '{ct}'"
+                )
+
     if not (service_doc := link_by_rel(links, "service-doc")):
-        warnings.append("/ : Link[rel=service-doc] must exist")
+        warnings.append("/ : Link[rel=service-doc] should exist")
     else:
         if service_doc.get("type") != "text/html":
             errors.append("service-doc type is not text/html")
@@ -293,7 +311,9 @@ def validate_core(
             lambda: r_service_doc.status_code == 200,
         )
 
-        if (ct := r_service_doc.headers.get("content-type")).startswith("text/html"):
+        if (ct := r_service_doc.headers.get("content-type", "")).startswith(
+            "text/html"
+        ):
             pass
         else:
             errors.append(
@@ -314,9 +334,31 @@ def validate_children(
 
 
 def validate_collections(
-    root_body: Dict[str, Any], warnings: List[str], errors: List[str]
+    root_body: Dict[str, Any], collection: str, warnings: List[str], errors: List[str]
 ) -> None:
-    print("Collections validation is not yet implemented.")
+    print("WARNING: Collections validation is not yet fully implemented.")
+
+    if not (data_link := link_by_rel(root_body["links"], "data")):
+        errors.append("[Collections] /: Link[rel=data] must href /collections")
+    else:
+        r = requests.get(f"{data_link['href']}/{collection}")
+        if not (items_link := link_by_rel(r.json()["links"], "items")):
+            errors.append(
+                f"[Collections] /collections/{collection} is missing link with rel='items'"
+            )
+        else:
+            collection_url = items_link["href"]
+            r = requests.get(collection_url)
+            if r.status_code != 200:
+                errors.append(
+                    f"[Collections] link rel='items' returned status code {r.status_code}"
+                )
+            try:
+                r.json()
+            except json.decoder.JSONDecodeError:
+                errors.append(
+                    f"[Collections] link rel='items' ({collection_url}) had json problem"
+                )
 
 
 def validate_features(
@@ -352,50 +394,52 @@ def validate_features(
     # todo: add this one somewhere
     #  "http://www.opengis.net/spec/ogcapi-features-1/1.0/conf/oas30",
 
-    links = root_body.get("links")
-    conformance = link_by_rel(links, "conformance")
+    root_links = root_body.get("links")
+    conformance = link_by_rel(root_links, "conformance")
     errors += validate(
         "/ Link[rel=conformance] must href /conformance",
-        lambda: conformance and conformance["href"].endswith("/conformance"),
+        lambda: conformance is not None
+        and conformance.get("href", "").endswith("/conformance"),
     )
 
-    r_conformance = requests.get(conformance["href"])
-    errors += validate(
-        "conformance must return 200", lambda: r_conformance.status_code == 200
-    )
-
-    if (ct := r_conformance.headers.get("content-type"), "").split(";")[
-        0
-    ] == "application/json":
-        pass
-    else:
-        errors.append(
-            f"conformance ({conformance}): must have content-type header 'application/json', actually '{ct}'"
+    if conformance:
+        r_conformance = requests.get(conformance["href"])
+        errors += validate(
+            "conformance must return 200", lambda: r_conformance.status_code == 200
         )
 
-    try:
-        conformance_json = r_conformance.json()
+        if (ct := r_conformance.headers.get("content-type", "")).split(";")[
+            0
+        ] == "application/json":
+            pass
+        else:
+            errors.append(
+                f"conformance ({conformance}): must have content-type header 'application/json', actually '{ct}'"
+            )
+
+        try:
+            conformance_json = r_conformance.json()
+            warnings += validate(
+                "Landing Page conforms to and conformance conformsTo must be the same",
+                lambda: set(root_body.get("conformsTo", []))
+                == set(conformance_json["conformsTo"]),
+            )
+        except json.decoder.JSONDecodeError:
+            errors.append(
+                f"service-desc ({conformance}): must return JSON, instead got non-JSON text"
+            )
+
+        errors += validate(
+            "/: Link[rel=data] must href /collections",
+            lambda: link_by_rel(root_links, "data") is not None,
+        )
+
+        # this is hard to figure out, since it's likely a mistake, but most apis can't undo it for
+        # backwards-compat reasons
         warnings += validate(
-            "Landing Page conforms to and conformance conformsTo must be the same",
-            lambda: set(root_body.get("conformsTo"), [])
-            == set(conformance_json["conformsTo"]),
+            "/ Link[rel=collections] is a non-standard relation. Use Link[rel=data instead]",
+            lambda: link_by_rel(root_links, "collections") is None,
         )
-    except json.decoder.JSONDecodeError:
-        errors.append(
-            f"service-desc ({conformance}): must return JSON, instead got non-JSON text"
-        )
-
-    errors += validate(
-        "/: Link[rel=data] must href /collections",
-        lambda: link_by_rel(links, "data") is not None,
-    )
-
-    # this is hard to figure out, since it's likely a mistake, but most apis can't undo it for
-    # backwards-compat reasons
-    warnings += validate(
-        "/ Link[rel=collections] is a non-standard relation. Use Link[rel=data instead]",
-        lambda: link_by_rel(links, "collections") is None,
-    )
 
     # if any(cc_features_fields_regex.fullmatch(x) for x in conforms_to):
     #     print("STAC API - Features - Fields extension conformance class found.")
@@ -416,6 +460,7 @@ def validate_features(
 def validate_item_search(
     root_body: Dict[str, Any],
     post: bool,
+    collection: str,
     conforms_to: List[str],
     warnings: List[str],
     errors: List[str],
@@ -455,13 +500,16 @@ def validate_item_search(
             f"Search ({search_url}): must return JSON, instead got non-JSON text"
         )
 
-    validate_item_searc_limit(search_url, post, errors)
-    validate_item_searc_bbox_xor_intersects(search_url, post, errors)
-    validate_item_searc_bbox(search_url, post, errors)
-    validate_item_searc_datetime(search_url, warnings, errors)
-    validate_item_searc_ids(search_url, post, warnings, errors)
-    validate_item_searc_collections(search_url, collections_url, post, errors)
-    validate_item_searc_intersects(search_url, post, errors)
+    validate_item_search_limit(search_url, post, errors)
+    validate_item_search_bbox_xor_intersects(search_url, post, errors)
+    validate_item_search_bbox(search_url, post, errors)
+    validate_item_search_datetime(search_url, warnings, errors)
+    validate_item_search_ids(search_url, post, warnings, errors)
+    validate_item_search_ids_does_not_override_all_other_params(
+        search_url, post, collection, warnings, errors
+    )
+    validate_item_search_collections(search_url, collections_url, post, errors)
+    validate_item_search_intersects(search_url, post, errors)
 
     # if any(cc_item_search_fields_regex.fullmatch(x) for x in conforms_to):
     #     print("STAC API - Item Search - Fields extension conformance class found.")
@@ -496,7 +544,7 @@ def validate_item_search_filter() -> None:
     pass
 
 
-def validate_item_searc_datetime(
+def validate_item_search_datetime(
     search_url: str, warnings: List[str], errors: List[str]
 ) -> None:
     # find an Item and try to use its datetime value in a query
@@ -540,7 +588,7 @@ def validate_item_searc_datetime(
     # todo: POST
 
 
-def validate_item_searc_bbox_xor_intersects(
+def validate_item_search_bbox_xor_intersects(
     search_url: str, post: bool, errors: List[str]
 ) -> None:
     r = requests.get(
@@ -562,7 +610,7 @@ def validate_item_searc_bbox_xor_intersects(
             )
 
 
-def validate_item_searc_intersects(
+def validate_item_search_intersects(
     search_url: str, post: bool, errors: List[str]
 ) -> None:
     intersects_params = [
@@ -606,7 +654,7 @@ def validate_item_searc_intersects(
                     )
 
 
-def validate_item_searc_bbox(search_url: str, post: bool, errors: List[str]) -> None:
+def validate_item_search_bbox(search_url: str, post: bool, errors: List[str]) -> None:
     # Valid GET query
     param = "100.0,0.0,105.0,1.0"
     r = requests.get(search_url, params={"bbox": param})
@@ -718,8 +766,8 @@ def validate_item_searc_bbox(search_url: str, post: bool, errors: List[str]) ->
                 )
 
 
-def validate_item_searc_limit(search_url: str, post: bool, errors: List[str]) -> None:
-    valid_limits = [1, 2, 10, 1000]
+def validate_item_search_limit(search_url: str, post: bool, errors: List[str]) -> None:
+    valid_limits = [1, 2, 10]  # todo: pull actual limits from service description
     for limit in valid_limits:
         # Valid GET query
         params = {"limit": limit}
@@ -762,7 +810,7 @@ def validate_item_searc_limit(search_url: str, post: bool, errors: List[str]) ->
                         f"POST Search with {params} returned non-json response: {r.text}"
                     )
 
-    invalid_limits = [-1, 0, 10001]
+    invalid_limits = [-1]  # todo: pull actual limits from service desc and test them
     for limit in invalid_limits:
         # Valid GET query
         params = {"limit": limit}
@@ -831,7 +879,52 @@ def _validate_search_ids_with_ids(
         )
 
 
-def validate_item_searc_ids(
+def _validate_search_ids_with_ids_no_override(
+    search_url: str, item: Dict[str, Any], post: bool, errors: List[str]
+) -> None:
+    bbox = item["bbox"]
+    get_params = {
+        "ids": item["id"],
+        "collections": item["collection"],
+        "bbox": f"{bbox[2] + 1},{bbox[3] + 1},{bbox[2] + 2},{bbox[3] + 2}",
+    }
+
+    r = requests.get(search_url, params=get_params)
+    try:
+        if len(r.json().get("features")) > 0:
+            errors.append(
+                "GET Search with ids and non-intersecting bbox returned results, indicating "
+                "the ids parameter is overriding the bbox parameter. All parameters are applied equally since "
+                "STAC API 1.0.0-beta.1"
+            )
+    except json.decoder.JSONDecodeError:
+        errors.append(
+            f"GET Search with {get_params} returned non-json response: {r.text}"
+        )
+
+    if post:
+        post_params = {
+            "ids": [item["id"]],
+            "collections": [item["collection"]],
+            "bbox": [bbox[2] + 1, bbox[3] + 1, bbox[2] + 2, bbox[3] + 2],
+        }
+
+        r = requests.post(search_url, json=post_params)
+
+        try:
+            if len(r.json().get("features")) > 0:
+                errors.append(
+                    "POST Search with ids and non-intersecting bbox returned results, indicating "
+                    "the ids parameter is overriding the bbox parameter. All parameters are applied equally since "
+                    "STAC API 1.0.0-beta.1"
+                )
+        except json.decoder.JSONDecodeError:
+            errors.append(
+                f"POST Search with {get_params} returned non-json response: {r.text}"
+            )
+
+
+def validate_item_search_ids(
     search_url: str, post: bool, warnings: List[str], errors: List[str]
 ) -> None:
     r = requests.get(f"{search_url}?limit=2")
@@ -848,6 +941,23 @@ def validate_item_searc_ids(
         warnings.append("Get Search with no parameters returned < 2 results")
 
 
+def validate_item_search_ids_does_not_override_all_other_params(
+    search_url: str, post: bool, collection: str, warnings: List[str], errors: List[str]
+) -> None:
+    # find one item that we can then query by id and a non-intersecting bbox to see if
+    # we still get the item as a result
+    if (
+        items := requests.get(
+            f"{search_url}?limit=1&bbox=20,20,21,21&collections={collection}"
+        )
+        .json()
+        .get("features")
+    ):
+        _validate_search_ids_with_ids_no_override(search_url, items[0], post, errors)
+    else:
+        warnings.append("Get Search with no parameters returned 0 results")
+
+
 def _validate_search_collections_request(
     r: requests.Response,
     coll_ids: List[str],
@@ -897,7 +1007,7 @@ def _validate_search_collections_with_ids(
         )
 
 
-def validate_item_searc_collections(
+def validate_item_search_collections(
     search_url: str, collections_url: Optional[str], post: bool, errors: List[str]
 ) -> None:
     if collections_url:
